@@ -24,8 +24,8 @@ pygtk.require('2.0')
 import os
 import gtk
 
-from hamster import dispatcher, storage, SHARED_DATA_DIR
-from hamster.Configuration import GconfStore
+from hamster import dispatcher, storage, SHARED_DATA_DIR, stuff
+from hamster.configuration import GconfStore
 
 def get_prev(selection, model):
     (model, iter) = selection.get_selected()
@@ -67,7 +67,7 @@ class ActivityStore(gtk.ListStore):
             
         self.clear()
 
-        if category_id == None:
+        if category_id is None:
             return
         
         activity_list = storage.get_activities(category_id)
@@ -89,8 +89,9 @@ class PreferencesEditor:
         ]
     
     
-    def __init__(self):
-        self.glade = gtk.glade.XML(os.path.join(SHARED_DATA_DIR, "preferences.glade"))
+    def __init__(self, parent = None):
+        self.parent = parent
+        self._gui = stuff.load_ui_file("preferences.ui")
         self.config = GconfStore.get_instance()
         self.window = self.get_widget('preferences_window')
 
@@ -139,7 +140,7 @@ class PreferencesEditor:
 
         self.load_config()
 
-        self.glade.signal_autoconnect(self)
+        self._gui.connect_signals(self)
 
         # Allow enable drag and drop of rows including row move
         self.activity_tree.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
@@ -178,6 +179,7 @@ class PreferencesEditor:
         self.get_widget("idle_track").set_active(self.config.get_timeout_enabled())
         self.get_widget("notify_interval").set_value(self.config.get_notify_interval())
         self.get_widget("keybinding").set_text(self.config.get_keybinding())
+        self.get_widget("notify_on_idle").set_active(self.config.get_notify_on_idle())
 
 
     def drag_data_get_data(self, treeview, context, selection, target_id,
@@ -292,7 +294,7 @@ class PreferencesEditor:
 
     def get_widget(self, name):
         """ skip one variable (huh) """
-        return self.glade.get_widget(name)
+        return self._gui.get_object(name)
 
     def get_store(self):
         """returns store, so we can add some watchers in case if anything changes"""
@@ -317,7 +319,7 @@ class PreferencesEditor:
                 return False
 
         if id == -2: #new category
-            id = storage.add_category(new_text)
+            id = storage.add_category(new_text.decode("utf-8"))
             model[path][0] = id
         else:
             storage.update_category(id, new_text)
@@ -338,7 +340,11 @@ class PreferencesEditor:
                 self.select_activity(activity['id'])
                 return False
         
-        model[path][0] = storage.add_activity(new_text, category_id)
+        
+        if id == -1: #new activity -> add
+            model[path][0] = storage.add_activity(new_text.decode("utf-8"), category_id)
+        else: #existing activity -> update
+            storage.update_activity(id, new_text.decode("utf-8"), category_id)
         model[path][1] = new_text
         return True
         
@@ -347,15 +353,19 @@ class PreferencesEditor:
         """ enables and disables action buttons depending on selected item """
         (model, iter) = selection.get_selected()
         id = 0
-        if iter == None:
+        if iter is None:
             self.activity_store.clear()
         else:
+            self.prev_selected_activity = None
+
             id = model[iter][0]
             self.activity_store.load(model[iter][0])
         
-        #do not allow to remove the unsorted category
-        self.get_widget('promote_activity').set_sensitive(False)
-        self.get_widget('demote_activity').set_sensitive(False)
+        #start with nothing
+        self.get_widget('activity_up').set_sensitive(False)
+        self.get_widget('activity_down').set_sensitive(False)
+        self.get_widget('activity_edit').set_sensitive(False)
+        self.get_widget('activity_remove').set_sensitive(False)
 
         return True
 
@@ -375,14 +385,18 @@ class PreferencesEditor:
         
         # treat any selected case
         unsorted_selected = self._get_selected_category() == -1
-        self.get_widget('promote_activity').set_sensitive(False)
-        self.get_widget('demote_activity').set_sensitive(False)
+        self.get_widget('activity_up').set_sensitive(False)
+        self.get_widget('activity_down').set_sensitive(False)
+
+        self.get_widget('activity_edit').set_sensitive(iter != None)
+        self.get_widget('activity_remove').set_sensitive(iter != None)
+        
         if iter != None and not unsorted_selected:
             first_item = model.get_path(iter) == (0,)
-            self.get_widget('promote_activity').set_sensitive(not first_item)
+            self.get_widget('activity_up').set_sensitive(not first_item)
 
-            last_item = model.iter_next(iter) == None
-            self.get_widget('demote_activity').set_sensitive(not last_item)
+            last_item = model.iter_next(iter) is None
+            self.get_widget('activity_down').set_sensitive(not last_item)
 
     def _del_selected_row(self, tree):
         selection = tree.get_selection()
@@ -446,14 +460,26 @@ class PreferencesEditor:
             self.prev_selected_category = path
         
 
+    def on_activity_remove_clicked(self, button):
+        self.remove_current_activity()
+
+    def on_activity_edit_clicked(self, button):
+        self.activityCell.set_property("editable", True)
+
+        selection = self.activity_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        path = model.get_path(iter)[0]
+        self.activity_tree.set_cursor(path, focus_column = self.activityColumn, start_editing = True)
+        
+
+
     """keyboard events"""
     def on_activity_list_key_pressed(self, tree, event_key):
         key = event_key.keyval
         selection = tree.get_selection()
         (model, iter) = selection.get_selected()
         if (event_key.keyval == gtk.keysyms.Delete):
-            storage.remove_activity(model[iter][0])
-            self._del_selected_row(tree)
+            self.remove_current_activity()
 
         elif key == gtk.keysyms.F2 :
             self.activityCell.set_property("editable", True)
@@ -462,7 +488,25 @@ class PreferencesEditor:
             #tree.grab_focus()
             #tree.set_cursor(path, start_editing = True)
 
+    def remove_current_activity(self):
+        selection = self.activity_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        storage.remove_activity(model[iter][0])
+        self._del_selected_row(self.activity_tree)
 
+
+    def on_category_remove_clicked(self, button):
+        self.remove_current_category()
+        
+    def on_category_edit_clicked(self, button):
+        self.categoryCell.set_property("editable", True)
+
+        selection = self.category_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        path = model.get_path(iter)[0]
+        self.category_tree.set_cursor(path, focus_column = self.categoryColumn, start_editing = True)
+        
+        
     def on_category_list_key_pressed(self, tree, event_key):
         key = event_key.keyval
         
@@ -473,10 +517,7 @@ class PreferencesEditor:
         (model, iter) = selection.get_selected()
 
         if  key == gtk.keysyms.Delete:
-            id = model[iter][0]
-            if id != -1:
-                storage.remove_category(id)
-                self._del_selected_row(tree)
+            self.remove_current_category()
         elif key == gtk.keysyms.F2:
             self.categoryCell.set_property("editable", True)
             path = model.get_path(iter)[0]
@@ -484,11 +525,19 @@ class PreferencesEditor:
             #tree.grab_focus()
             #tree.set_cursor(path, start_editing = True)
 
+    def remove_current_category(self):
+        selection = self.category_tree.get_selection()
+        (model, iter) = selection.get_selected()
+        id = model[iter][0]
+        if id != -1:
+            storage.remove_category(id)
+            self._del_selected_row(self.category_tree)
+
     def on_preferences_window_key_press(self, widget, event):
         # ctrl+w means close window
         if (event.keyval == gtk.keysyms.w \
             and event.state & gtk.gdk.CONTROL_MASK):
-            self.window.destroy()
+            self.close_window()
 
         # escape can mean several things
         if event.keyval == gtk.keysyms.Escape:
@@ -500,10 +549,10 @@ class PreferencesEditor:
                 self.categoryCell.set_property("editable", False)
                 return
 
-            self.window.destroy()     
+            self.close_window()     
 
     """button events"""
-    def on_add_category_clicked(self, button):
+    def on_category_add_clicked(self, button):
         """ appends row, jumps to it and allows user to input name """
         
         new_category = self.category_store.insert_before(self.category_store.unsorted_category,
@@ -516,7 +565,7 @@ class PreferencesEditor:
                                          start_editing = True)
 
 
-    def on_add_activity_clicked(self, button):
+    def on_activity_add_clicked(self, button):
         """ appends row, jumps to it and allows user to input name """
         category_id = self._get_selected_category()
         
@@ -530,31 +579,43 @@ class PreferencesEditor:
                                          focus_cell = None,
                                          start_editing = True)
 
-    def on_remove_activity_clicked(self, button):
+    def on_activity_remove_clicked(self, button):
         removable_id = self._del_selected_row(self.activity_tree)
         storage.remove_activity(removable_id)
 
-    def on_promote_activity_clicked(self, button):
+    def on_activity_up_clicked(self, button):
         (model, iter) = self.selection.get_selected()
 
         #previous item
         prev_iter = get_prev(self.selection, model)
-        storage.swap_activities(model[iter][0], model[prev_iter][0])
+        storage.swap_activities(model[iter][0], model[iter][3],
+                                model[prev_iter][0], model[prev_iter][3])
         model.move_before(iter, prev_iter)
 
         self.activity_changed(self.selection, model)
 
-    def on_demote_activity_clicked(self, button):
+    def on_activity_down_clicked(self, button):
         (model, iter) = self.selection.get_selected()
 
         next_iter = model.iter_next(iter)
-        storage.swap_activities(model[iter][0], model[next_iter][0])
+        storage.swap_activities(model[iter][0], model[iter][3],
+                                model[next_iter][0], model[next_iter][3])
         self.activity_store.move_after(iter, next_iter)
 
         self.activity_changed(self.selection, model)
 
     def on_close_button_clicked(self, button):
-        self.window.destroy()
+        self.close_window()
+
+    def on_close(self, widget, event):
+        self.close_window()        
+    
+    def close_window(self):
+        if not self.parent:
+            gtk.main_quit()
+        else:
+            self.window.destroy()
+            return False
 
     def on_shutdown_track_toggled(self, checkbox):
         self.config.set_stop_on_shutdown(checkbox.get_active())
@@ -562,6 +623,9 @@ class PreferencesEditor:
     def on_idle_track_toggled(self, checkbox):
         self.config.set_timeout_enabled(checkbox.get_active())
 
+    def on_notify_on_idle_toggled(self, checkbox):
+        self.config.set_notify_on_idle(checkbox.get_active())
+
     def on_notify_interval_format_value(self, slider, value):
         if value <=120:
             # notify interval slider value label
@@ -573,7 +637,9 @@ class PreferencesEditor:
         return label
     
     def on_notify_interval_value_changed(self, scale):
-        self.config.set_notify_interval(int(scale.get_value()))
+        value = int(scale.get_value())
+        self.config.set_notify_interval(value)
+        self.get_widget("notify_on_idle").set_sensitive(value <= 120)
     
     def on_keybinding_changed(self, textbox):
         self.config.set_keybinding(textbox.get_text())
